/* --------------------------------------------------------------------------------------------
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_1 = require("vscode");
const protocol_proposed_1 = require("./protocol.proposed");
class WorkspaceFoldersFeature {
    constructor(_client) {
        this._client = _client;
        this._listeners = new Map();
    }
    get messages() {
        return protocol_proposed_1.DidChangeWorkspaceFolders.type;
    }
    fillClientCapabilities(capabilities) {
        capabilities.workspace = capabilities.workspace || {};
        let workspace = capabilities.workspace;
        workspace.workspaceFolders = true;
    }
    initialize(_documentSelector, _capabilities) {
        let client = this._client;
        client.onRequest(protocol_proposed_1.GetWorkspaceFolders.type, () => {
            let folders = vscode_1.workspace.workspaceFolders;
            if (folders === void 0) {
                return null;
            }
            let result = folders.map((folder) => {
                return this.asProtocol(folder);
            });
            return result;
        });
        client.onRequest(protocol_proposed_1.GetWorkspaceFolder.type, (uri) => {
            let folder = vscode_1.workspace.getWorkspaceFolder(client.protocol2CodeConverter.asUri(uri));
            if (folder === void 0) {
                return null;
            }
            return this.asProtocol(folder);
        });
    }
    register(_message, data) {
        let id = data.id;
        let disposable = vscode_1.workspace.onDidChangeWorkspaceFolders((event) => {
            let params = {
                event: {
                    added: event.added.map(folder => this.asProtocol(folder)),
                    removed: event.removed.map(folder => this.asProtocol(folder))
                }
            };
            this._client.sendNotification(protocol_proposed_1.DidChangeWorkspaceFolders.type, params);
        });
        this._listeners.set(id, disposable);
        let folders = vscode_1.workspace.workspaceFolders;
        if (folders) {
            this._client.sendNotification(protocol_proposed_1.DidChangeWorkspaceFolders.type, {
                event: {
                    added: folders.map(folder => this.asProtocol(folder)),
                    removed: []
                }
            });
        }
    }
    unregister(id) {
        let disposable = this._listeners.get(id);
        if (disposable === void 0) {
            return;
        }
        this._listeners.delete(id);
        disposable.dispose();
    }
    dispose() {
        for (let disposable of this._listeners.values()) {
            disposable.dispose();
        }
        this._listeners.clear();
    }
    asProtocol(workspaceFolder) {
        if (workspaceFolder === void 0) {
            return null;
        }
        return { uri: this._client.code2ProtocolConverter.asUri(workspaceFolder.uri), name: workspaceFolder.name };
    }
}
exports.WorkspaceFoldersFeature = WorkspaceFoldersFeature;
class GetConfigurationFeature {
    constructor(_client) {
        this._client = _client;
    }
    fillClientCapabilities(capabilities) {
        capabilities.workspace = capabilities.workspace || {};
        let workspace = capabilities.workspace;
        workspace.getConfiguration = true;
    }
    initialize(_documentSelector, _capabilities) {
        let client = this._client;
        client.onRequest(protocol_proposed_1.GetConfigurationRequest.type, (params) => {
            let result = [];
            let scopes = params.scopeUris;
            if (scopes === void 0 || scopes === null) {
                result.push(this.getConfigurations(undefined, params.sections));
            }
            else {
                for (let uri of scopes) {
                    result.push(this.getConfigurations(this._client.protocol2CodeConverter.asUri(uri), params.sections));
                }
            }
            return result;
        });
    }
    getConfigurations(resource, sections) {
        let result = [];
        if (sections === void 0 || sections === null) {
            result.push(resource, undefined);
        }
        else {
            for (let section of sections) {
                result.push(this.getConfiguration(resource, section));
            }
        }
        return result;
    }
    getConfiguration(resource, section) {
        let result = null;
        if (section) {
            let index = section.lastIndexOf('.');
            if (index === -1) {
                result = vscode_1.workspace.getConfiguration(undefined, resource).get(section);
            }
            else {
                let config = vscode_1.workspace.getConfiguration(section.substr(0, index));
                if (config) {
                    result = config.get(section.substr(index + 1));
                }
            }
        }
        else {
            let config = vscode_1.workspace.getConfiguration(undefined, resource);
            result = {};
            for (let key of Object.keys(config)) {
                if (config.has(key)) {
                    result[key] = config.get(key);
                }
            }
        }
        if (!result) {
            return null;
        }
        return result;
    }
}
exports.GetConfigurationFeature = GetConfigurationFeature;
function ProposedProtocol(client) {
    let result = [];
    result.push(new WorkspaceFoldersFeature(client));
    result.push(new GetConfigurationFeature(client));
    return result;
}
exports.ProposedProtocol = ProposedProtocol;
